# Perform the analyses with 'constant' pair QGt but randomised male SH
# Base the code on that used for holding male SH constant but shuffling QGt

# BEWARE - this script uses a different version of the 'real data' - the female names are prefixed with 'fem' rather than 'M'

# Revised 2017-08-01 for Dryad
#  - tidied, streamlined, and better annotated

#--------------------------------------------------------------------------------------------
# Clear R
rm(list = ls())

# Load necessary packages
library(lme4)
library(MuMIn)
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
####  BRING IN DATA  ########

# Set WD
setwd("FILEPATH")

# bring in data
real.data <- read.csv("11_HFC_coreData_focalFamilies_adjustedPrefixes.csv")
sim.data <- read.csv("11_varyingMaleSH_butHoldQGt_dyads.csv")
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
## Code for Nakagawa Rsq for LMM
## http://jslefche.wordpress.com/2013/03/13/r2-for-linear-mixed-effects-models/
## Updated by us, as original was written for lme4 in R v2

rsquared.lme=function(modlist) {
  #Iterate over each model in the list
  do.call(rbind,lapply(modlist,function(i) {
    #For models fit using lm
    if(class(i)=="lm") {
      Rsquared.mat=data.frame(Class=class(i),Marginal=summary(i)$r.squared,
                              Conditional=NA,AIC=AIC(i)) } 
    #For models fit using lme4
    else if(class(i)=="mer" | class(i)=="merMod" | class(i)=="merLmerTest") {
      #Get variance of fixed effects by multiplying coefficients by design matrix
      VarF=var(as.vector(fixef(i) %*% t(i@X))) 
      #Get variance of random effects by extracting variance components
      VarRand=colSums(do.call(rbind,lapply(VarCorr(i),function(j) j[1])))
      #Get residual variance
      VarResid=attr(VarCorr(i),"sc")^2
      #Calculate marginal R-squared (fixed effects/total variance)
      Rm=VarF/(VarF+VarRand+VarResid)
      #Calculate conditional R-squared (fixed effects+random effects/total variance)
      Rc=(VarF+VarRand)/(VarF+VarRand+VarResid)
      #Bind R^2s into a matrix and return with AIC values
      Rsquared.mat=data.frame(Class=class(i),Marginal=Rm,Conditional=Rc,
                              AIC=AIC(update(i,REML=F))) } 
    #For models fit using NEWER VERSION of lme4 WITH lmerTest
    else if(class(i)=="lmerMod" | class(i)=="merModLmerTest") {
      #Get variance of fixed effects by multiplying coefficients by design matrix
      VarF=var(as.vector(fixef(i) %*% t(i@pp$X))) 
      #Get variance of random effects by extracting variance components
      VarRand=colSums(do.call(rbind,lapply(VarCorr(i),function(j) j[1])))
      #Get residual variance
      VarResid=attr(VarCorr(i),"sc")^2
      #Calculate marginal R-squared (fixed effects/total variance)
      Rm=VarF/(VarF+VarRand+VarResid)
      #Calculate conditional R-squared (fixed effects+random effects/total variance)
      Rc=(VarF+VarRand)/(VarF+VarRand+VarResid)
      #Bind R^2s into a matrix and return with AIC values
      Rsquared.mat=data.frame(Class=class(i),Marginal=Rm,Conditional=Rc,
                              AIC=AIC(update(i,REML=F))) } 
    #For model fit using nlme  
    else if(class(i)=="lme") {
      #Get design matrix of fixed effects from model
      Fmat=model.matrix(eval(i$call$fixed)[-2],i$data)
      #Get variance of fixed effects by multiplying coefficients by design matrix
      VarF=var(as.vector(fixef(i) %*% t(Fmat)))
      #Get variance of random effects by extracting variance components
      VarRand=sum(suppressWarnings(as.numeric(VarCorr(i)[rownames(VarCorr(i))!=
                                                           "Residual",1])),na.rm=T)
      #Get residual variance
      VarResid=as.numeric(VarCorr(i)[rownames(VarCorr(i))=="Residual",1])
      #Calculate marginal R-squared (fixed effects/total variance)
      Rm=VarF/(VarF+VarRand+VarResid)
      #Calculate conditional R-squared (fixed effects+random effects/total variance)
      Rc=(VarF+VarRand)/(VarF+VarRand+VarResid)
      #Bind R^2s into a matrix and return with AIC values
      Rsquared.mat=data.frame(Class=class(i),Marginal=Rm,Conditional=Rc,
                              AIC=AIC(update(i,method="ML")))
    } else { print("Function requires models of class lm, lme, mer, or merMod") 
    } } ) ) }
#  rsquared.lme(list( ENTER MODEL NAMES HERE ))
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
##  Collinearity diagnostics

vif.mer <- function (fit) {
  ## adapted from rms::vif
  
  v <- vcov(fit)
  nam <- names(fixef(fit))
  
  ## exclude intercepts
  ns <- sum(1 * (nam == "Intercept" | nam == "(Intercept)"))
  if (ns > 0) {
    v <- v[-(1:ns), -(1:ns), drop = FALSE]
    nam <- nam[-(1:ns)]
  }
  
  d <- diag(v)^0.5
  v <- diag(solve(v/(d %o% d)))
  names(v) <- nam
  v
}

kappa.mer <- function (fit,
                       scale = TRUE, center = FALSE,
                       add.intercept = TRUE,
                       exact = FALSE) {
  X <- fit@pp$X
  nam <- names(fixef(fit))
  
  ## exclude intercepts
  nrp <- sum(1 * (nam == "(Intercept)"))
  if (nrp > 0) {
    X <- X[, -(1:nrp), drop = FALSE]
    nam <- nam[-(1:nrp)]
  }
  
  if (add.intercept) {
    X <- cbind(rep(1), scale(X, scale = scale, center = center))
    kappa(X, exact = exact)
  } else {
    kappa(scale(X, scale = scale, center = scale), exact = exact)
  }
}
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
####  EXAMINE/REFINE DATA  ########

str(real.data)
str(sim.data)


# Mean-centre variables
real.data$lay.date.C <- scale(real.data$lay.date, center = TRUE, scale = FALSE)
real.data$incub.dur.C <- scale(real.data$incub.dur.1, center = TRUE, scale = FALSE)
real.data$male.SH.C <- scale(real.data$male.SH, center = TRUE, scale = FALSE)
real.data$pair.QGt.C <- scale(real.data$pair.QGt, center = TRUE, scale = FALSE)

# Make quadratic date
real.data$lay.date.C.SQ <- real.data$lay.date.C^2


# Get SH-QGt correlation 
REAL.SH.QGt.cor.data <- real.data[c("pair.id", "male.SH", "pair.QGt")]
REAL.SH.QGt.cor.data <- unique(REAL.SH.QGt.cor.data)
(REAL.SH.QGt.cor <- cor(REAL.SH.QGt.cor.data$male.SH, REAL.SH.QGt.cor.data$pair.QGt))
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
# DO THE lme4 ANALYSIS ON THE REAL DATASET
# Null model
REAL.mod.null <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
	+ (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.null.AICc <- AICc(REAL.mod.null))
(REAL.mod.null.vif <- vif.mer(REAL.mod.null))
(REAL.mod.null.kappa <- kappa.mer(REAL.mod.null))


# Make SH only
REAL.mod.SH <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
	+ male.SH.C + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.SH.AICc <- AICc(REAL.mod.SH))
(REAL.mod.SH.vif <- vif.mer(REAL.mod.SH))
(REAL.mod.SH.kappa <- kappa.mer(REAL.mod.SH))


# Parental relatedness/similarity
REAL.mod.QGt <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
	+ pair.QGt.C + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.QGt.AICc <- AICc(REAL.mod.QGt))
(REAL.mod.QGt.vif <- vif.mer(REAL.mod.QGt))
(REAL.mod.QGt.kappa <- kappa.mer(REAL.mod.QGt))


# Both, no interaction
REAL.mod.BOTH <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
	+ male.SH.C + pair.QGt.C + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.BOTH.AICc <- AICc(REAL.mod.BOTH))
(REAL.mod.BOTH.vif <- vif.mer(REAL.mod.BOTH))
(REAL.mod.BOTH.kappa <- kappa.mer(REAL.mod.BOTH))


# Both, with interaction
REAL.mod.INT <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
	+ male.SH.C*pair.QGt.C + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.INT.AICc <- AICc(REAL.mod.INT))
(REAL.mod.INT.vif <- vif.mer(REAL.mod.INT))
(REAL.mod.INT.kappa <- kappa.mer(REAL.mod.INT))

REAL.mod.INT.summary <- summary(REAL.mod.INT)
REAL.mod.INT.coefs <- REAL.mod.INT.summary$coefficients

str(REAL.mod.INT.summary)

# Get R2
(REAL.Rsq <- rsquared.lme(c(REAL.mod.null, REAL.mod.SH, REAL.mod.QGt, REAL.mod.BOTH, REAL.mod.INT)))


###   COLLECT THE METRICS FOR LATER COMPARISON WITH SHUFFLED DATA
### WHAT DO I NEED TO COLLECT FROM EACH SHUFFLE?
# diagnostics - vif, kappa, cor(SH,QGt)
# summary metrics - dAICc, dRSq
# model parameters - slopes, SE, t


OUT.NAMES <- c("d.AICc.INT", "d.RSq.INT", "INT.slope", "INT.SE", "INT.t", 
	"kappa.INT", "VIF.max.INT", "SH.QGt.cor",
	"QGt.AICc", "BOTH.AICc", "INT.AICc",
	"QGt.RSq", "BOTH.RSq", "INT.RSq")

REAL.OUT.DATA <- c(REAL.mod.INT.AICc - REAL.mod.null.AICc,		# delta AICc of interaction model against 'null'
	REAL.Rsq$Marginal[5] - REAL.Rsq$Marginal[1],			# delta marginal R-sq of interaction
	REAL.mod.INT.coefs["male.SH.C:pair.QGt.C","Estimate"],	# coefficient of interaction term
	REAL.mod.INT.coefs["male.SH.C:pair.QGt.C","Std. Error"],	# SE for interaction term
	REAL.mod.INT.coefs["male.SH.C:pair.QGt.C","t value"],		# t-value for interaction term
	REAL.mod.INT.kappa, max(REAL.mod.INT.vif), REAL.SH.QGt.cor,		# QC diagnostics
	REAL.mod.QGt.AICc, REAL.mod.BOTH.AICc, REAL.mod.INT.AICc,		# AICc values for the three varying models
	REAL.Rsq$Marginal[3], REAL.Rsq$Marginal[4], REAL.Rsq$Marginal[5])		# RSq

names(REAL.OUT.DATA) <- OUT.NAMES
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
# Function to generate new mal.SH and QGt columns
# Pass it a female ID and the simulated data
# Return the pair QGt and SH for random male from that females list of candidates
# Will be used within SHUFFLE.SH() function

get.QGt.SH <- function(fem.id, sim.data){
  sim.subset <- subset(sim.data, female.id == fem.id)
  random.line <- sim.subset[sample(nrow(sim.subset),1),]
  new.data <- c(random.line[[3]], random.line[[5]])		# QGt ad mal SH
  return(new.data)
}
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
# Function to 'shuffle' male SH and return new dataframe with shuffled SH (and QGt)
# Will be used to make dataframe to pass to MAIN()
SHUFFLE.SH <- function(REAL, SIM){
  # make new QGt and male.SH data - one new entry per family
  ID.list <- REAL$female.id		# female IDs, with repeats
  ID.list <- table(ID.list)		# no repeats
  ID.list <- names(ID.list)		# names, no repeats
  shuffled.data.no.reps <- data.frame(mat.or.vec(length(ID.list), 3))
  for(i in 1:length(ID.list)){
    shuffled.data.no.reps[i,1] <- ID.list[i]
    shuffled.data.no.reps[i,2:3] <- get.QGt.SH(ID.list[i], SIM)
  }
  
  # put in replication for repeat pairs
  shuffled.genetics <- mat.or.vec(nrow(REAL),2)
  for(i in 1:nrow(REAL)){
    pair.genetics <- subset(shuffled.data.no.reps, X1 == REAL$female.id[i])
    shuffled.genetics[i,1:2] <- c(pair.genetics[[2]], pair.genetics[[3]])
  }
  
  # Put the shuffled male SH (and slightly tweaked QGt) back into the real (SHUFFLED) data (after copying real data)
  SHUFFLED.DATA <- REAL
  SHUFFLED.DATA$pair.QGt <- shuffled.genetics[,1]
  SHUFFLED.DATA$male.SH <- shuffled.genetics[,2]
  
  # mean-centre the new male.SH and QGt
  SHUFFLED.DATA$male.SH.C <- scale(SHUFFLED.DATA$male.SH, center = TRUE, scale = FALSE)
  SHUFFLED.DATA$pair.QGt.C <- scale(SHUFFLED.DATA$pair.QGt, center = TRUE, scale = FALSE)
  
  return(SHUFFLED.DATA)
}
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
###   MAIN FUNCTION START
#  Pass it the output dataframe from SHUFFLE.SH

MAIN <- function(SHUFFLED.D){
  # Get SH-QGt correlation 
  SHUFF.SH.QGt.cor.data <- SHUFFLED.D[c("pair.id", "male.SH", "pair.QGt")]
  SHUFF.SH.QGt.cor.data <- unique(SHUFF.SH.QGt.cor.data)
  SHUFF.SH.QGt.cor <- cor(SHUFF.SH.QGt.cor.data$male.SH, SHUFF.SH.QGt.cor.data$pair.QGt)
  
  
  # New models - as for real/observed data
  SHUFF.mod.null <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
  	+ (1|pair.id), data=SHUFFLED.D, REML = FALSE)
  
  SHUFF.mod.null.AICc <- AICc(SHUFF.mod.null)
  SHUFF.mod.null.vif <- vif.mer(SHUFF.mod.null)
  SHUFF.mod.null.kappa <- kappa.mer(SHUFF.mod.null)
  
  
  SHUFF.mod.SH <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
  	+ male.SH.C + (1|pair.id), data=SHUFFLED.D, REML = FALSE)
  
  SHUFF.mod.SH.AICc <- AICc(SHUFF.mod.SH)
  SHUFF.mod.SH.vif <- vif.mer(SHUFF.mod.SH)
  SHUFF.mod.SH.kappa <- kappa.mer(SHUFF.mod.SH)
  
  
  SHUFF.mod.QGt <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
  	+ pair.QGt.C + (1|pair.id), data=SHUFFLED.D, REML = FALSE)
  
  SHUFF.mod.QGt.AICc <- AICc(SHUFF.mod.QGt)
  SHUFF.mod.QGt.vif <- vif.mer(SHUFF.mod.QGt)
  SHUFF.mod.QGt.kappa <- kappa.mer(SHUFF.mod.QGt)
  
  
  SHUFF.mod.BOTH <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
  	+ male.SH.C + pair.QGt.C + (1|pair.id), data=SHUFFLED.D, REML = FALSE)

  SHUFF.mod.BOTH.AICc <- AICc(SHUFF.mod.BOTH)
  SHUFF.mod.BOTH.vif <- vif.mer(SHUFF.mod.BOTH)
  SHUFF.mod.BOTH.kappa <- kappa.mer(SHUFF.mod.BOTH)
  
  
  SHUFF.mod.INT <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
  	+ male.SH.C*pair.QGt.C + (1|pair.id), data=SHUFFLED.D, REML = FALSE)

  SHUFF.mod.INT.AICc <- AICc(SHUFF.mod.INT)
  SHUFF.mod.INT.vif <- vif.mer(SHUFF.mod.INT)
  SHUFF.mod.INT.kappa <- kappa.mer(SHUFF.mod.INT)
  
  SHUFF.mod.INT.summary <- summary(SHUFF.mod.INT)
  SHUFF.mod.INT.coefs <- SHUFF.mod.INT.summary$coefficients
  
  
  SHUFF.Rsq <- rsquared.lme(c(SHUFF.mod.null, SHUFF.mod.SH, SHUFF.mod.QGt, SHUFF.mod.BOTH, SHUFF.mod.INT))
  
  
  # Gather summaries
  OUT.LINE <- c(SHUFF.mod.INT.AICc - SHUFF.mod.null.AICc,	# delta AICc of interaction model against 'null'
  	SHUFF.Rsq$Marginal[5] - SHUFF.Rsq$Marginal[1],		# delta marginal R-sq of interaction
  	SHUFF.mod.INT.coefs["male.SH.C:pair.QGt.C","Estimate"],	# coefficient of interaction term
  	SHUFF.mod.INT.coefs["male.SH.C:pair.QGt.C","Std. Error"],	# SE for interaction term
  	SHUFF.mod.INT.coefs["male.SH.C:pair.QGt.C","t value"],	# t-value for interaction term
  	SHUFF.mod.INT.kappa, max(SHUFF.mod.INT.vif), SHUFF.SH.QGt.cor,	# QC diagnostics
	SHUFF.mod.QGt.AICc, SHUFF.mod.BOTH.AICc, SHUFF.mod.INT.AICc,	# AICc values for the three varying models
	SHUFF.Rsq$Marginal[3], SHUFF.Rsq$Marginal[4], SHUFF.Rsq$Marginal[5])	# RSq
  
  return(OUT.LINE)
}
###   END OF MAIN FUNCTION
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
# Run the function 1000 times
n.reps <- 1000

# first rep, then dataframe + names
SHUFFLED.D <- SHUFFLE.SH(real.data, sim.data)
SHUFF.OUT.DATA <- rbind(REAL.OUT.DATA, MAIN(SHUFFLED.D))

SHUFF.OUT.DATA <- data.frame(SHUFF.OUT.DATA)


# Loop the rest
for(i in 1:(n.reps-1)){
  SHUFFLED.D <- SHUFFLE.SH(real.data, sim.data)
  SHUFF.OUT.DATA <- rbind(SHUFF.OUT.DATA, MAIN(SHUFFLED.D))
}

#SAVE
#write.csv(SHUFF.OUT.DATA, "11_varyMaleSH_butHoldQGt_OUTPUTS_1000.csv", row.names=FALSE)
#--------------------------------------------------------------------------------------------


#--------------------------------------------------------------------------------------------
# ASSESS THE OBSERVED VALUES AGAINST THE SHUFFLES

SHUFF.OUT.DATA <- SHUFF.OUT.DATA[2:nrow(SHUFF.OUT.DATA),]

# dAICc - one-tailed; how often is shuffled lower than observed
c(REAL.OUT.DATA["d.AICc.INT"], (sum(SHUFF.OUT.DATA$d.AICc.INT <= REAL.OUT.DATA["d.AICc.INT"])) / n.reps)

# dRsq - one-tailed; how often is shuffled greater than observed
c(REAL.OUT.DATA["d.RSq.INT"], (sum(SHUFF.OUT.DATA$d.RSq.INT >= REAL.OUT.DATA["d.RSq.INT"])) / n.reps)

# interaction slope - what is distance between observed and mean of shuffled, in SDs of shuffled?
(mean(SHUFF.OUT.DATA$INT.slope) - REAL.OUT.DATA["INT.slope"]) / sd(SHUFF.OUT.DATA$INT.slope)

# how does kappa compare; one-tailed, how often is shuffled greater than observed
c(REAL.OUT.DATA["kappa.INT"], (sum(SHUFF.OUT.DATA$kappa.INT >= REAL.OUT.DATA["kappa.INT"])) / n.reps)

# how does max.VIF compare; one-tailed, how often is shuffled greater than observed
c(REAL.OUT.DATA["VIF.max.INT"], (sum(SHUFF.OUT.DATA$VIF.max.INT >= REAL.OUT.DATA["VIF.max.INT"])) / n.reps)

# SH-QGt correlation: observed, mean, and Probability of shuffled Cor being more negative
SH.QGt.cor.summary <- c(REAL.OUT.DATA["SH.QGt.cor"], 
	mean(SHUFF.OUT.DATA$SH.QGt.cor),
	(sum(SHUFF.OUT.DATA$SH.QGt.cor <= REAL.OUT.DATA["SH.QGt.cor"])) / n.reps)
names(SH.QGt.cor.summary) <- c("obs.cor", "mean.shuff.cor", "P.one.tail")
SH.QGt.cor.summary

# Does the strength of the SH-QGt correlation predict signficance (delta AICc)?
plot(SHUFF.OUT.DATA$d.AICc.INT ~ SHUFF.OUT.DATA$SH.QGt.cor)
points(x = REAL.OUT.DATA["SH.QGt.cor"], y = REAL.OUT.DATA["d.AICc.INT"],
	pch = 4, col = "red", lwd = 2)
#--------------------------------------------------------------------------------------------
