####################################################################################
# Outputs from null simulations of main analysis
# Compare observed values to simulated nulls

#-------------------------------------------------------------------------------------------
# Clear R
rm(list=ls())

# Load packages
library(lme4)
library(MuMIn)

# Set WD
setwd("FILEPATH")

# Load observed data
real.data <- read.csv("00_HFC_coreData_focalFamilies.csv")
#str(real.data)

# Load simulation data
summaries.truegenos <- read.csv("12_nullSimulationSummaries_MLH_truegenos.csv")
summaries.P90 <- read.csv("12_nullSimulationSummaries_MLH_P90.csv")
str(summaries.truegenos)
str(summaries.P90)
#-------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------
## Code for Nakagawa Rsq for LMM
## http://jslefche.wordpress.com/2013/03/13/r2-for-linear-mixed-effects-models/

rsquared.lme=function(modlist) {
  #Iterate over each model in the list
  do.call(rbind,lapply(modlist,function(i) {
    #For models fit using lm
    if(class(i)=="lm") {
      Rsquared.mat=data.frame(Class=class(i),Marginal=summary(i)$r.squared,
                              Conditional=NA,AIC=AIC(i)) } 
    #For models fit using lme4
    else if(class(i)=="mer" | class(i)=="merMod" | class(i)=="merLmerTest") {
      #Get variance of fixed effects by multiplying coefficients by design matrix
      VarF=var(as.vector(fixef(i) %*% t(i@X))) 
      #Get variance of random effects by extracting variance components
      VarRand=colSums(do.call(rbind,lapply(VarCorr(i),function(j) j[1])))
      #Get residual variance
      VarResid=attr(VarCorr(i),"sc")^2
      #Calculate marginal R-squared (fixed effects/total variance)
      Rm=VarF/(VarF+VarRand+VarResid)
      #Calculate conditional R-squared (fixed effects+random effects/total variance)
      Rc=(VarF+VarRand)/(VarF+VarRand+VarResid)
      #Bind R^2s into a matrix and return with AIC values
      Rsquared.mat=data.frame(Class=class(i),Marginal=Rm,Conditional=Rc,
                              AIC=AIC(update(i,REML=F))) } 
    #For models fit using NEWER VERSION of lme4
    else if(class(i)=="lmerMod") {
      #Get variance of fixed effects by multiplying coefficients by design matrix
      VarF=var(as.vector(fixef(i) %*% t(i@pp$X))) 
      #Get variance of random effects by extracting variance components
      VarRand=colSums(do.call(rbind,lapply(VarCorr(i),function(j) j[1])))
      #Get residual variance
      VarResid=attr(VarCorr(i),"sc")^2
      #Calculate marginal R-squared (fixed effects/total variance)
      Rm=VarF/(VarF+VarRand+VarResid)
      #Calculate conditional R-squared (fixed effects+random effects/total variance)
      Rc=(VarF+VarRand)/(VarF+VarRand+VarResid)
      #Bind R^2s into a matrix and return with AIC values
      Rsquared.mat=data.frame(Class=class(i),Marginal=Rm,Conditional=Rc,
                              AIC=AIC(update(i,REML=F))) } 
    #For model fit using nlme  
    else if(class(i)=="lme") {
      #Get design matrix of fixed effects from model
      Fmat=model.matrix(eval(i$call$fixed)[-2],i$data)
      #Get variance of fixed effects by multiplying coefficients by design matrix
      VarF=var(as.vector(fixef(i) %*% t(Fmat)))
      #Get variance of random effects by extracting variance components
      VarRand=sum(suppressWarnings(as.numeric(VarCorr(i)[rownames(VarCorr(i))!=
                                                           "Residual",1])),na.rm=T)
      #Get residual variance
      VarResid=as.numeric(VarCorr(i)[rownames(VarCorr(i))=="Residual",1])
      #Calculate marginal R-squared (fixed effects/total variance)
      Rm=VarF/(VarF+VarRand+VarResid)
      #Calculate conditional R-squared (fixed effects+random effects/total variance)
      Rc=(VarF+VarRand)/(VarF+VarRand+VarResid)
      #Bind R^2s into a matrix and return with AIC values
      Rsquared.mat=data.frame(Class=class(i),Marginal=Rm,Conditional=Rc,
                              AIC=AIC(update(i,method="ML")))
    } else { print("Function requires models of class lm, lme, mer, or merMod") 
    } } ) ) }
#  rsquared.lme(list( ENTER MODEL NAMES HERE ))
#-------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------
##  Collinearity diagnostics

vif.mer <- function (fit) {
  ## adapted from rms::vif
  
  v <- vcov(fit)
  nam <- names(fixef(fit))
  
  ## exclude intercepts
  ns <- sum(1 * (nam == "Intercept" | nam == "(Intercept)"))
  if (ns > 0) {
    v <- v[-(1:ns), -(1:ns), drop = FALSE]
    nam <- nam[-(1:ns)]
  }
  
  d <- diag(v)^0.5
  v <- diag(solve(v/(d %o% d)))
  names(v) <- nam
  v
}

kappa.mer <- function (fit,
                       scale = TRUE, center = FALSE,
                       add.intercept = TRUE,
                       exact = FALSE) {
  X <- fit@pp$X
  nam <- names(fixef(fit))
  
  ## exclude intercepts
  nrp <- sum(1 * (nam == "(Intercept)"))
  if (nrp > 0) {
    X <- X[, -(1:nrp), drop = FALSE]
    nam <- nam[-(1:nrp)]
  }
  
  if (add.intercept) {
    X <- cbind(rep(1), scale(X, scale = scale, center = center))
    kappa(X, exact = exact)
  } else {
    kappa(scale(X, scale = scale, center = scale), exact = exact)
  }
}
#-------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------
# mean-centre variables
real.data$lay.date.C <- scale(real.data$lay.date, center = TRUE, scale = FALSE)
real.data$incub.dur.C <- scale(real.data$incub.dur.1, center = TRUE, scale = FALSE)
real.data$male.SH.C <- scale(real.data$male.SH, center = TRUE, scale = FALSE)
real.data$pair.QGt.C <- scale(real.data$pair.QGt, center = TRUE, scale = FALSE)

# make quadratic variables
real.data$lay.date.C.SQ <- real.data$lay.date.C^2
#head(real.data)


# Get SH-QGt correlation 
REAL.SH.QGt.cor.data <- real.data[c("pair.id", "male.SH", "pair.QGt")]
REAL.SH.QGt.cor.data <- unique(REAL.SH.QGt.cor.data)
(REAL.SH.QGt.cor <- cor(REAL.SH.QGt.cor.data$male.SH, REAL.SH.QGt.cor.data$pair.QGt, use="complete.obs"))
#-------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------
# DO THE lme4 ANALYSIS ON THE REAL DATASET
# Null model
REAL.mod.null <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
                      + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.null.AICc <- AICc(REAL.mod.null))
(REAL.mod.null.vif <- vif.mer(REAL.mod.null))
(REAL.mod.null.kappa <- kappa.mer(REAL.mod.null))


# Male SH only
REAL.mod.SH <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
                    + male.SH.C + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.SH.AICc <- AICc(REAL.mod.SH))
(REAL.mod.SH.vif <- vif.mer(REAL.mod.SH))
(REAL.mod.SH.kappa <- kappa.mer(REAL.mod.SH))


# Parental relatedness/similarity
REAL.mod.QGt <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
                     + pair.QGt.C + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.QGt.AICc <- AICc(REAL.mod.QGt))
(REAL.mod.QGt.vif <- vif.mer(REAL.mod.QGt))
(REAL.mod.QGt.kappa <- kappa.mer(REAL.mod.QGt))


# Both, no interaction
REAL.mod.BOTH <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
                      + male.SH.C + pair.QGt.C + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.BOTH.AICc <- AICc(REAL.mod.BOTH))
(REAL.mod.BOTH.vif <- vif.mer(REAL.mod.BOTH))
(REAL.mod.BOTH.kappa <- kappa.mer(REAL.mod.BOTH))


# Both, with interaction
REAL.mod.INT <- lmer(TO.SEA ~ lay.date.C + lay.date.C.SQ + incub.dur.C
                     + male.SH.C*pair.QGt.C + (1|pair.id), data=real.data, REML = FALSE)

(REAL.mod.INT.AICc <- AICc(REAL.mod.INT))
(REAL.mod.INT.vif <- vif.mer(REAL.mod.INT))
(REAL.mod.INT.kappa <- kappa.mer(REAL.mod.INT))

REAL.mod.INT.summary <- summary(REAL.mod.INT)
REAL.mod.INT.coefs <- REAL.mod.INT.summary$coefficients


# Get R2
(REAL.Rsq <- rsquared.lme(c(REAL.mod.null, REAL.mod.SH, REAL.mod.QGt, REAL.mod.BOTH, REAL.mod.INT)))
anova(REAL.mod.null, REAL.mod.INT)
anova(REAL.mod.BOTH, REAL.mod.INT)


###   COLLECT THE METRICS FOR LATER COMPARISON WITH SHUFFLED DATA
### WHAT DO I NEED TO COLLECT?
# diagnostics - vif, kappa, cor(SH,QGt, use="complete.obs")
# summary metrics - dAICc, dRSq
# model parameters - slopes, SE, t

OUT.NAMES <- c("d.AICc.INT", "d.RSq.INT", "INT.slope", "INT.SE", "INT.t", 
               "kappa.INT", "VIF.max.INT", "SH.QGt.cor",
               "QGt.AICc", "BOTH.AICc", "INT.AICc",
               "QGt.RSq", "BOTH.RSq", "INT.RSq")

REAL.OUT.DATA <- c(REAL.mod.INT.AICc - REAL.mod.null.AICc,			# delta AICc of interaction model against 'null'
                   REAL.Rsq$Marginal[5] - REAL.Rsq$Marginal[1],			# delta marginal R-sq of interaction
                   REAL.mod.INT.coefs["male.SH.C:pair.QGt.C","Estimate"],	# coefficient of interaction term
                   REAL.mod.INT.coefs["male.SH.C:pair.QGt.C","Std. Error"],	# SE for interaction term
                   REAL.mod.INT.coefs["male.SH.C:pair.QGt.C","t value"],		# t-value for interaction term
                   REAL.mod.INT.kappa, max(REAL.mod.INT.vif), REAL.SH.QGt.cor,	# QC diagnostics
                   REAL.mod.QGt.AICc, REAL.mod.BOTH.AICc, REAL.mod.INT.AICc,			# AICc values for the three varying models
                   REAL.Rsq$Marginal[3], REAL.Rsq$Marginal[4], REAL.Rsq$Marginal[5])		# RSq

names(REAL.OUT.DATA) <- OUT.NAMES
#-------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------
#-------------------------------------------------------------------------------------------
# THE KEY METRICS FOR THE PAPER

# Focal observed metrics: dAICc, RSq.INT, and interaction slope
observed.dAICc <- REAL.OUT.DATA[1]
observed.RSq.INT <- REAL.OUT.DATA[2]
observed.int.slope <- REAL.OUT.DATA[3]


#str(summaries.truegenos)
#str(summaries.P90)

#~~~~~
# One-tailed P-value for dAICc.INT of P90 reconstructions
(sum(ifelse(summaries.P90$d.AICc.INT <= observed.dAICc, 1, 0)))/nrow(summaries.P90)

# The same test, but using the true genotypes instead of reconstructed genotypes
(sum(ifelse(summaries.truegenos$d.AICc.INT <= observed.dAICc, 1, 0)))/nrow(summaries.truegenos)

# P-values are the same - no meaningful bias
# SIG - main result stands

# Get mean and SD of DIFFERENCES for dAICc
dAICc.DIFFS <- summaries.P90$d.AICc.INT - summaries.truegenos$d.AICc.INT
c(mean(dAICc.DIFFS), sd(dAICc.DIFFS))
# Tiny difference
#~~~~~

#~~~~~
# One-tailed P-value for change in marginal RSq.INT for P90, complete datasets
(sum(ifelse(summaries.P90$d.RSq.INT >= observed.RSq.INT, 1, 0)))/nrow(summaries.P90)

# For equivalent sets in truegenos
(sum(ifelse(summaries.truegenos$d.RSq.INT >= observed.RSq.INT, 1, 0)))/nrow(summaries.truegenos)

# P-values are the same - no meaningful bias

# Means and SD of DIFFERENCES
RSq.DIFFS <- summaries.P90$d.RSq.INT - summaries.truegenos$d.RSq.INT
c(mean(RSq.DIFFS), sd(RSq.DIFFS))
# Tiny difference
#~~~~~

#~~~~~
# One-tailed P-value for slope of interaction term, P90, complete datasets
(sum(ifelse(summaries.P90$INT.slope >= observed.int.slope, 1, 0)))/nrow(summaries.P90)

# And for truegenos but reduced to complete P90 batches
(sum(ifelse(summaries.truegenos$INT.slope >= observed.int.slope, 1, 0)))/nrow(summaries.truegenos)

# Marginal change in P-value, but still sig

# Means and SD of DIFFERENCES
INT.slope.DIFFS <- summaries.P90$INT.slope - summaries.truegenos$INT.slope
c(mean(INT.slope.DIFFS), sd(INT.slope.DIFFS))
# Tiny difference
#~~~~~

#~~~~~
# g2
G2.DIFFS <- summaries.P90$g2.P90 - summaries.P90$g2.true
c(mean(G2.DIFFS), sd(G2.DIFFS))
#~~~~~

#~~~~~
# Pearson
pearson.DIFFS <- summaries.P90$SH.QGt.cor - summaries.truegenos$SH.QGt.cor
c(mean(pearson.DIFFS), sd(pearson.DIFFS))
#~~~~~

#~~~~~
# VIF and kappa
VIF.bias <- summaries.P90$VIF.max.INT - summaries.truegenos$VIF.max.INT
kappa.bias <- summaries.P90$kappa.INT - summaries.truegenos$kappa.INT
c(mean(VIF.bias), sd(VIF.bias))
c(mean(kappa.bias), sd(kappa.bias))
#~~~~~

#~~~~~
# Male SH & pair QGt
mal.SH.bias <- summaries.P90$mal.SH.bias.mean
mal.QGt.bias <- summaries.P90$pair.QGt.bias.mean
#~~~~~

# Gather these
GATHERED.BIAS <- data.frame(rbind(c(mean(mal.SH.bias), sd(mal.SH.bias)),
				c(mean(mal.QGt.bias), sd(mal.QGt.bias)),
				c(mean(dAICc.DIFFS), sd(dAICc.DIFFS)),
				c(mean(RSq.DIFFS), sd(RSq.DIFFS)),
				c(mean(INT.slope.DIFFS), sd(INT.slope.DIFFS)),
				c(mean(G2.DIFFS), sd(G2.DIFFS)),
				c(mean(pearson.DIFFS), sd(pearson.DIFFS)),
				c(mean(VIF.bias), sd(VIF.bias)),
				c(mean(kappa.bias), sd(kappa.bias)) ))
names(GATHERED.BIAS) <- c("mean.diff", "sd.diff")
rownames(GATHERED.BIAS) <- c("mal.SH", "pair.QGt", "dAICc", "dRSq", "INT.slope", "g2", "pearson", "VIF", "kappa")
GATHERED.BIAS$Z.diff <- round(GATHERED.BIAS$mean.diff / GATHERED.BIAS$sd.diff, 3)
GATHERED.BIAS$mean.diff <- round(GATHERED.BIAS$mean.diff, 6)
GATHERED.BIAS$sd.diff <- round(GATHERED.BIAS$sd.diff, 6)
GATHERED.BIAS

# Bias to male SH, but not affecting 

# Get male bias as percentage of 'true' mean
summaries.truegenos$mal.SH
names(summaries.P90)

# One-tailed P-value for SH-QGt correlation; P90, complete
observed.pearson <- REAL.OUT.DATA[8]
(sum(ifelse(summaries.P90$SH.QGt.cor <= observed.pearson, 1, 0)))/nrow(summaries.P90)

# True equivalent
(sum(ifelse(summaries.truegenos$SH.QGt.cor <= observed.pearson, 1, 0)))/nrow(summaries.truegenos)

# Check correlation between strength of MLH-QGt correlation and effect sizes
cor.test(summaries.P90$d.AICc.INT, summaries.P90$SH.QGt.cor, method="spearman")
cor.test(summaries.P90$d.RSq.INT, summaries.P90$SH.QGt.cor, method="spearman")
cor.test(summaries.P90$INT.slope, summaries.P90$SH.QGt.cor, method="spearman")

#-------------------------------------------------------------------------------------------
#-------------------------------------------------------------------------------------------
