###########################################################
# Examine the single-locus null simulation models
# Data contain P-values for respective likelihood ratio tests, the number of single loci with t > 2, and the slope for each locus

#----------------------------------------------------------------------------------
# Clear R
rm(list=ls())

# Set WD
setwd("FILEPATH")

# Load data
data.null.P90 <- read.csv("13_nullSimulations_SLH_P90.csv")
data.null.true <- read.csv("13_nullSimulations_SLH_truegenos.csv")

names(data.null.P90)
names(data.null.true)
nrow(data.null.P90)
#----------------------------------------------------------------------------------


#----------------------------------------------------------------------------------
# Examine distribution of P-values v MLH for 'true'
#hist(data.null.true$P.mal.SLH.MLH)
#hist(data.null.true$P.fem.SLH.MLH)
#hist(data.null.true$P.simQGt.SLH.MLH)
#hist(data.null.true$P.simAvg.SLH.MLH)

# How many P < 0.05 for 'true', SLH v MLH
sum(data.null.true$P.mal.SLH.MLH <= 0.05) / nrow(data.null.true)
sum(data.null.true$P.fem.SLH.MLH <= 0.05) / nrow(data.null.true)
sum(data.null.true$P.simQGt.SLH.MLH <= 0.05) / nrow(data.null.true)
sum(data.null.true$P.simAvg.SLH.MLH <= 0.05) / nrow(data.null.true)
# STUPID-LOOKING - clearly needs adjusting

# And for P < 0.01?
sum(data.null.true$P.mal.SLH.MLH <= 0.01) / nrow(data.null.true)
sum(data.null.true$P.fem.SLH.MLH <= 0.01) / nrow(data.null.true)
sum(data.null.true$P.simQGt.SLH.MLH <= 0.01) / nrow(data.null.true)
sum(data.null.true$P.simAvg.SLH.MLH <= 0.01) / nrow(data.null.true)
# Also stupid

# And for P < 0.001? Just for a laugh!
sum(data.null.true$P.mal.SLH.MLH <= 0.001) / nrow(data.null.true)
sum(data.null.true$P.fem.SLH.MLH <= 0.001) / nrow(data.null.true)
sum(data.null.true$P.simQGt.SLH.MLH <= 0.001) / nrow(data.null.true)
sum(data.null.true$P.simAvg.SLH.MLH <= 0.001) / nrow(data.null.true)
# Very stupid
#----------------------------------------------------------------------------------


#----------------------------------------------------------------------------------
# What about the P90 reconstructions? 
sum(data.null.P90$P.mal.SLH.MLH <= 0.05) / nrow(data.null.P90)
sum(data.null.P90$P.fem.SLH.MLH <= 0.05) / nrow(data.null.P90)
sum(data.null.P90$P.simQGt.SLH.MLH <= 0.05) / nrow(data.null.P90)
sum(data.null.P90$P.simAvg.SLH.MLH <= 0.05) / nrow(data.null.P90)
# All slightly elevated
#----------------------------------------------------------------------------------


#----------------------------------------------------------------------------------
# What P-value would constitute sig for 'true'?
quantile(data.null.true$P.mal.SLH.MLH, 0.05)
quantile(data.null.true$P.fem.SLH.MLH, 0.05)
quantile(data.null.true$P.simQGt.SLH.MLH, 0.05)
quantile(data.null.true$P.simAvg.SLH.MLH, 0.05)
# Well, the observed data aren't anything near that!

# And P90?
quantile(data.null.P90$P.mal.SLH.MLH, 0.05)
quantile(data.null.P90$P.fem.SLH.MLH, 0.05)
quantile(data.null.P90$P.simQGt.SLH.MLH, 0.05)
quantile(data.null.P90$P.simAvg.SLH.MLH, 0.05)
# Very demanding!
#----------------------------------------------------------------------------------


#====================================================================================
# Where do observed values sit?
obs.fem <- 0.216
obs.mal <- 0.009
obs.sim <- 0.001

# For truegenos
sum(data.null.true$P.mal.SLH.MLH <= obs.mal) / nrow(data.null.true)	# P = 0.491
sum(data.null.true$P.simQGt.SLH.MLH <= obs.sim) / nrow(data.null.true)	# P = 0.146
sum(data.null.true$P.fem.SLH.MLH <= obs.fem) / nrow(data.null.true)	# P = 0.866

# For P90
sum(data.null.P90$P.mal.SLH.MLH <= obs.mal) / nrow(data.null.P90)		# P = 0.506
sum(data.null.P90$P.simQGt.SLH.MLH <= obs.sim) / nrow(data.null.P90)	# P = 0.127
sum(data.null.P90$P.fem.SLH.MLH <= obs.fem) / nrow(data.null.P90)		# P = 0.898
#====================================================================================