# Script to test for resconstruction bias to male SH
#   - look at effects of number of offspring, P-threshold, and minimum number of loci allowed in

# UPDATED 2017-08-02
#  - tidy and annotate for Dryad

#---------------------------------------------------------------------------------------------------
# Clear R
rm(list=ls())

# Load packages
library(lme4)
library(MuMIn)

# Set WD
setwd("FILEPATH")

# Load data
RECON.MALE.DATA <- read.csv("14a_exploreReconstructionParams_biasToSH.csv")

str(RECON.MALE.DATA)
#---------------------------------------------------------------------------------------------------


#---------------------------------------------------------------------------------------------------
# Prepare predictors as factors

# Turn dad.id to factor
RECON.MALE.DATA$DAD.ID <- as.factor(RECON.MALE.DATA$DAD.ID)

# Copy dataset, and turn predictors to factors
RECON.MALE.DATA.FACTORS <- RECON.MALE.DATA
RECON.MALE.DATA.FACTORS$P.thresh <- as.factor(RECON.MALE.DATA.FACTORS$P.thresh)
RECON.MALE.DATA.FACTORS$n.loci <- as.factor(RECON.MALE.DATA.FACTORS$n.loci)

# Make three-way factor
# Offspring number, P-threshold, and locus threshold
three.way <- vector()
for(i in 1:nrow(RECON.MALE.DATA.FACTORS)){
  if(RECON.MALE.DATA.FACTORS[i,1] < 10){
    three.way <- c(three.way, paste(0, RECON.MALE.DATA.FACTORS[i,1], "_P", RECON.MALE.DATA.FACTORS[i,2], "_L", RECON.MALE.DATA.FACTORS[i,3], sep=""))
  }else{
    three.way <- c(three.way, paste(RECON.MALE.DATA.FACTORS[i,1], "_P", RECON.MALE.DATA.FACTORS[i,2], "_L", RECON.MALE.DATA.FACTORS[i,3], sep=""))
}}
RECON.MALE.DATA.FACTORS <- cbind(RECON.MALE.DATA.FACTORS, three.way)

# Turn offspring number to factor
RECON.MALE.DATA.FACTORS$n.offsp <- as.factor(RECON.MALE.DATA.FACTORS$n.offsp)

str(RECON.MALE.DATA.FACTORS)
#---------------------------------------------------------------------------------------------------


#---------------------------------------------------------------------------------------------------
# Basic plots

# Number of offspring
plot(SH.DIFF ~ n.offsp, data = RECON.MALE.DATA.FACTORS, las=1,
	main = "All simulated data",
	xlab = "Number of offspring in family sample",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

# Minimum per-locus confidence
plot(SH.DIFF ~ P.thresh, data = RECON.MALE.DATA.FACTORS, las=1,
	main = "All simulated data",
	xlab = "Per-locus reconstruction confidence threshold (%)",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

# Minimum number of loci required for a father to be included
plot(SH.DIFF ~ n.loci, data = RECON.MALE.DATA.FACTORS, las=1,
	main = "All simulated data",
	xlab = "Minimum number of loci in father genotype",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

# How well do real and reconstructed SH correlate?
plot(SH.DIFF ~ real.SH, data = RECON.MALE.DATA.FACTORS, las=1,
	main = "All simulated data",
	xlab = "Real paternal SH",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

# Definitely forget P95 and n.loci < 29
#---------------------------------------------------------------------------------------------------


#---------------------------------------------------------------------------------------------------
# Subset it for n.loci =29, 
subset.29 <- RECON.MALE.DATA.FACTORS[RECON.MALE.DATA.FACTORS$n.loci==29,]

# First, look at effect of P when N.loci = 29
plot(SH.DIFF ~ P.thresh, data = subset.29, las=1,
	main = "Minimum loci = 29",
	xlab = "Per-locus reconstruction confidence threshold (%)",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

# Look at offspring
plot(SH.DIFF ~ n.offsp, data = subset.29, las=1,
	main = "Minimum loci = 29",
	xlab = "Number of offspring in family sample",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

# Look at real.SH
plot(SH.DIFF ~ real.SH, data = subset.29, las=1,
	main = "Minimum loci = 29",
	xlab = "Real paternal SH",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)


# Keeping n.loci=29, analyse P85 and P90 separately
plot(SH.DIFF ~ n.offsp, data = subset(subset.29, P.thresh==85), las=1,
	ylim = c(-0.095, 0.131),
	main = "P-threshold = 0.85; minimum loci = 29",
	xlab = "Number of offspring in family sample",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

plot(SH.DIFF ~ n.offsp, data = subset(subset.29, P.thresh==90), las=1,
	ylim = c(-0.095, 0.131),
	main = "P-threshold = 0.90; minimum loci = 29",
	xlab = "Number of offspring in family sample",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)


# And real.SH effect?
plot(SH.DIFF ~ real.SH, data = subset(subset.29, P.thresh==85), las=1,
	main = "P-threshold = 0.85; minimum loci = 29",
	xlab = "Number of offspring in family sample",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

# And real.SH effect?
plot(SH.DIFF ~ real.SH, data = subset(subset.29, P.thresh==90), las=1,
	main = "P-threshold = 0.85; minimum loci = 29",
	xlab = "Number of offspring in family sample",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)
#---------------------------------------------------------------------------------------------------


#---------------------------------------------------------------------------------------------------
# Does parental similarity affect bias in reconstructing SH?
#   - as well as effects of both parental genotypes
# Include dad as random factor

# Basic models
mod.full.1 <- lmer(SH.DIFF ~ n.offsp + P.thresh + n.loci + (1|DAD.ID), data=RECON.MALE.DATA.FACTORS, REML=FALSE)
summary(mod.full.1)

mod.full.2 <- lmer(SH.DIFF ~ n.offsp + P.thresh + n.loci + real.SH + (1|DAD.ID), data=RECON.MALE.DATA.FACTORS, REML=FALSE)
summary(mod.full.2)	# Hmmm - real.SH is predictor of bias - as SH goes up, bias comes down


mod.P90.L29.1 <- lmer(SH.DIFF ~ n.offsp + (1|DAD.ID), data=subset(subset.29, P.thresh==90), REML=FALSE)
summary(mod.P90.L29.1)

mod.P90.L29.2 <- lmer(SH.DIFF ~ n.offsp + real.SH + (1|DAD.ID), data=subset(subset.29, P.thresh==90), REML=FALSE)
summary(mod.P90.L29.2)	# real.SH is still a predictor of bias
#---------------------------------------------------------------------------------------------------


#---------------------------------------------------------------------------------------------------
# Do the data look any better if plotted as mean for each family at each level?
# Start with subset.29 & only P90

subset.29.90 <- subset(subset.29, P.thresh==90)
head(subset.29.90)
n.offsp.range <- c(7, 10, 12, 15, 17, 20, 40)

# The next bit goes easier if the ranges and IDs are turned back to numbers
subset.29.90$n.offsp <- as.numeric(as.character(subset.29.90$n.offsp))
subset.29.90$DAD.ID <- as.numeric(as.character(subset.29.90$DAD.ID))

NEW.DATA.90 <- vector()
for(i in 1:length(n.offsp.range)){
  sub.1 <- subset(subset.29.90, n.offsp==n.offsp.range[i])
  sub.1.males <- as.numeric(names(table(sub.1$DAD.ID)))
  for(j in 1:length(sub.1.males)){
    sub.2 <- subset(sub.1, DAD.ID == sub.1.males[j])
    new.row <- c(sub.2$n.offsp[1], sub.2$DAD.ID[1], sub.2$real.SH[1], mean(sub.2$SH.DIFF))
    NEW.DATA.90 <- rbind(NEW.DATA.90, new.row)
}}

# Dataframe it, name it, turn n.off back to factors
NEW.DATA.90 <- data.frame(NEW.DATA.90)
names(NEW.DATA.90) <- c("n.offsp", "DAD.ID", "real.SH", "SH.DIFF")
NEW.DATA.90$n.offsp <- as.factor(NEW.DATA.90$n.offsp)


plot(SH.DIFF ~ n.offsp, data = NEW.DATA.90, las=1,
	ylim = c(-0.095, 0.131),
	main = "P-threshold = 0.90; minimum loci = 29",
	xlab = "Number of offspring in family sample",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

subset.29.85 <- subset(subset.29, P.thresh==85)
head(subset.29.85)
n.offsp.range <- c(7, 10, 12, 15, 17, 20, 40)

# The next bit goes easier if the ranges and IDs are turned back to numbers
subset.29.85$n.offsp <- as.numeric(as.character(subset.29.85$n.offsp))
subset.29.85$DAD.ID <- as.numeric(as.character(subset.29.85$DAD.ID))

NEW.DATA.85 <- vector()
for(i in 1:length(n.offsp.range)){
  sub.1 <- subset(subset.29.85, n.offsp==n.offsp.range[i])
  sub.1.males <- as.numeric(names(table(sub.1$DAD.ID)))
  for(j in 1:length(sub.1.males)){
    sub.2 <- subset(sub.1, DAD.ID == sub.1.males[j])
    new.row <- c(sub.2$n.offsp[1], sub.2$DAD.ID[1], sub.2$real.SH[1], mean(sub.2$SH.DIFF))
    NEW.DATA.85 <- rbind(NEW.DATA.85, new.row)
}}

# Dataframe it, name it, turn n.off back to factors
NEW.DATA.85 <- data.frame(NEW.DATA.85)
names(NEW.DATA.85) <- c("n.offsp", "DAD.ID", "real.SH", "SH.DIFF")
NEW.DATA.85$n.offsp <- as.factor(NEW.DATA.85$n.offsp)


plot(SH.DIFF ~ n.offsp, data = NEW.DATA.85, las=1,
	ylim = c(-0.095, 0.131),
	main = "P-threshold = 0.85; minimum loci = 29",
	xlab = "Number of offspring in family sample",
	ylab = "Bias in estimation of male SH")
lines(x=c(0,10), y=c(0,0), lty=2, col="red", lwd=2)
#---------------------------------------------------------------------------------------------------
